package code;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLDecoder;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSocketFactory;

import org.json.JSONException;

import com.sun.net.httpserver.Headers;
import com.sun.net.httpserver.HttpsExchange;

public class Helper {

	private static String keycloakRealm = "test00";
	private static String keycloakURL = "http://localhost:8080/";
	private static String cloudappURL = "https://localhost:3002/";
	private static String selfURL = "https://localhost:3001";

	public static void sendCors(HttpsExchange exchange, int stato, String response) throws IOException {
		Headers headers = exchange.getResponseHeaders();
		headers.add("Access-Control-Allow-Origin", "*");// se non mettiamo *, viene errore CORS //http://localhost:3001/secured/domains
		headers.add("Access-Control-Allow-Methods", "GET, POST, OPTIONS");
		headers.add("Access-Control-Allow-Headers", "Content-Type,Authorization");
		if(response == null){
			exchange.sendResponseHeaders(stato, -1);
			return;
		}
		byte[] responseBytes = response.getBytes();
		exchange.sendResponseHeaders(200, responseBytes.length);
		OutputStream os = exchange.getResponseBody();
		os.write(responseBytes);
		os.close();
	}

	public static void sendCors(HttpsExchange exchange, int stato) throws IOException {
		sendCors(exchange, stato, null);
	}

	public static void sendCors(HttpsExchange exchange, String response) throws IOException {
		sendCors(exchange, 200, response);
	}

	public static String checkTokenGetUser(HttpsExchange exchange) {
		String token = exchange.getRequestHeaders().get("Authorization").get(0).substring(7);// taglio bearer
		String[] tokSplit = token.split("[.]");
		if(tokSplit.length != 3) return null;
		String signature = tokSplit[2];
		try {
			return TokenHandler.verificaToken(token, signature);
		} catch (NoSuchAlgorithmException | IOException | JSONException e) {
			e.printStackTrace();
			return null;
		}
	}

	public static Map<String, Object> parseQuery(String query) throws UnsupportedEncodingException {
		if(query == null) throw new IllegalArgumentException("Query string can't be null");

		Map<String, Object> result = new HashMap<String, Object>();
		String pairs[] = query.split("[&]");
		for (String pair : pairs) {
			String param[] = pair.split("[=]");
			String key = null;
			String value = null;
			if (param.length > 0) {
				key = URLDecoder.decode(param[0], System.getProperty("file.encoding"));
			}

			if (param.length > 1) {
				value = URLDecoder.decode(param[1], System.getProperty("file.encoding"));
			}

			if (result.containsKey(key)) {
				Object obj = result.get(key);
				if (obj instanceof List<?>) {
					List<String> values = (List<String>) obj;
					values.add(value);
				} else if (obj instanceof String) {
					List<String> values = new ArrayList<String>();
					values.add((String) obj);
					values.add(value);
					result.put(key, values);
				}
			} else {
				result.put(key, value);
			}
		}
		return result;
	}

	public static String readBody(InputStream requestBody) {
		int request;
		StringBuffer response = new StringBuffer();
		try {
			while ((request = requestBody.read()) != -1)
				response.append(Character.toString((char) request));
		} catch (IOException e) {
			e.printStackTrace();
		}
		return response.toString();
	}

	public static HttpsURLConnection sendMessageToCloudapp(String path, String message) throws IOException {
		try {
			URL url = new URL(getCloudappURL() + (path.startsWith("/") ? path.substring(1) : path));
			System.out.println(url);
			HttpsURLConnection connection = (HttpsURLConnection) url.openConnection();
			System.out.println("Connection opened");
			// set the connection to work with SSL
			// connection.setSSLSocketFactory((SSLSocketFactory) SSLSocketFactory.getDefault());

			/*
				-Djavax.net.ssl.keyStoreType=pkcs12
				-Djavax.net.ssl.trustStoreType=jks
				-Djavax.net.ssl.keyStore=clientcertificate.p12
				-Djavax.net.ssl.trustStore=gridserver.keystore
				-Djavax.net.debug=ssl # very verbose debug
				-Djavax.net.ssl.keyStorePassword=$PASS
				-Djavax.net.ssl.trustStorePassword=$PASS
			System.setProperty("javax.net.ssl.keyStoreType", "pkcs12");
			System.setProperty("javax.net.ssl.keyStore", "../lig.keystore");
			System.setProperty("javax.net.ssl.keyStorePassword", "miapasswd");
			System.setProperty("javax.net.ssl.trustStore", "../lig.keystore");
			System.setProperty("javax.net.ssl.trustStorePassword", "miapasswd");
			System.setProperty("javax.net.debug", "ssl");
			*/

			System.out.println("SSL set");
			setConnectionSettings(connection,"POST");
			System.out.println("Connection settings set");
			DataOutputStream outStream = new DataOutputStream(connection.getOutputStream());//inserimento param in call
			System.out.println("Output stream opened");
			outStream.writeBytes(message);
			outStream.flush();
			outStream.close();
			return connection;
		} catch (Exception e) {
			//HERE remove this test
			e.printStackTrace();
			return null;
		}
	}

	public static void setConnectionSettings(HttpsURLConnection connection, String method) throws IOException{
		connection.setRequestMethod(method);
		connection.setRequestProperty("Content-Type", "application/json");
		connection.setRequestProperty("Accept", "application/json");
		connection.setDoOutput(true);
		connection.setConnectTimeout(5000);
		connection.setReadTimeout(5000);

	}

	public static String getResponseFromConnection(HttpsURLConnection con) throws IOException {
		BufferedReader reader = new BufferedReader(new InputStreamReader(con.getInputStream()));
		String inputLine;
		StringBuffer content = new StringBuffer();
		while ((inputLine = reader.readLine()) != null) {
			content.append(inputLine);
		}
		reader.close();
		return content.toString();
	}

	public static String getKeycloakRealm(){
		return keycloakRealm;
	}

	public static String getKeycloakURL(){
		return keycloakURL;
	}

	public static String getCloudappURL(){
		return cloudappURL;
	}

	public static String getSelfURL(){
		return selfURL;
	}

}
